@@ -1,6 +1,8 @@
 
-# Copyright (c) 2005 Nate Wiger <nate@wiger.org>. All Rights Reserved.
-# Use "perldoc CGI::FormBuilder::Multi" to read full documentation.
+###########################################################################
+# Copyright (c) Nate Wiger http://nateware.com. All Rights Reserved.
+# Please visit http://formbuilder.org for tutorials, support, and examples.
+###########################################################################
 
 package CGI::FormBuilder::Multi;
 
@@ -95,17 +97,19 @@ CGI::FormBuilder::Multi - Create multi-page FormBuilder forms
 =cut
 
 use strict;
+use warnings;
+no  warnings 'uninitialized';
 
-our $VERSION = '3.0302';
+use CGI::FormBuilder;
+use CGI::FormBuilder::Util;
+
+our $VERSION = '3.20';
 
 our %DEFAULT = (
     pagename => '_page',
     navbar   => 0,
 );
 
-use CGI::FormBuilder;
-use CGI::FormBuilder::Util;
-
 sub new {
     my $mod = shift;
     my $class = ref($mod) || $mod;
@@ -116,11 +120,18 @@ sub new {
     while (ref $_[0]) {
         push @forms, shift;
     }
-    puke "Must specify at least one form to ::Multi" unless @forms;
 
     # Remaining options are form opts
     my %opt  = arghash(@_);
 
+    # If no forms, and specified number of pages, use that instead
+    if ($opt{pages}) {
+        puke "Can't specify pages and form hashrefs" if @forms;
+        my $p = 0;
+        push @forms, {} while $p++ < $opt{pages};
+    }
+    puke "Must specify at least one form or 'pages' option for ::Multi" unless @forms;
+
     # Check for CGI params
     # This is duplicated code straight out of FormBuilder.pm,
     # but it's needed here as well so we can get our _page
@@ -281,6 +292,14 @@ C<header> (which will likely be the same), and then override
 individual settings like C<fields> and C<validate> on a per-form
 basis.
 
+If you do not wish to specify any options for your forms, you
+can instead just specify the C<pages> option, for example:
+
+    my $multi = CGI::FormBuilder::Multi->new(pages => 3);
+
+With this approach, you will have to dynamically assemble each
+page as you come to them. The mailing list can help.
+
 The L</"SYNOPSIS"> above is very representative of typical usage.
 
 =head2 form()
@@ -343,11 +362,11 @@ L<CGI::FormBuilder>
 
 =head1 REVISION
 
-$Id: Multi.pm,v 1.25 2006/02/24 01:42:29 nwiger Exp $
+$Id: Multi.pm 100 2007-03-02 18:13:13Z nwiger $
 
 =head1 AUTHOR
 
-Copyright (c) 2005-2006 Nate Wiger <nate@wiger.org>. All Rights Reserved.
+Copyright (c) L<Nate Wiger|http://nateware.com>. All Rights Reserved.
 
 This module is free software; you may copy this under the terms of
 the GNU General Public License, or the Artistic License, copies of
