@@ -1,6 +1,8 @@
 
-# Copyright (c) 2005 Nathan Wiger <nate@wiger.org>. All Rights Reserved.
-# Use "perldoc CGI::FormBuilder::Source::File" to read full documentation.
+###########################################################################
+# Copyright (c) Nate Wiger http://nateware.com. All Rights Reserved.
+# Please visit http://formbuilder.org for tutorials, support, and examples.
+###########################################################################
 
 package CGI::FormBuilder::Source::File;
 
@@ -21,11 +23,15 @@ CGI::FormBuilder::Source::File - Initialize FormBuilder from external file
 
 use Carp;
 use strict;
-
-our $VERSION = '3.0302';
+use warnings;
+no  warnings 'uninitialized';
 
 use 5.006; # or later
 use CGI::FormBuilder::Util;
+use utf8;
+
+
+our $VERSION = '3.20';
 
 # Begin "real" code
 sub new {
@@ -52,6 +58,7 @@ sub parse {
 
     my $refield = 0;
     my @file;
+    my $utf8 = 0;   # parse file as utf8
 
     debug 1, "parsing $file as input source";
     if (ref $file eq 'SCALAR') {
@@ -68,8 +75,11 @@ sub parse {
     my $inval = 0;
     for (@file) {
         next if /^\s*$/ || /^\s*#/;     # blanks and comments
+        next if /^\s*\[\%\s*\#|^\s*-*\%\]/;   # TT comments too
         chomp;
         my($term, $line) = split /\s*:\s*/, $_, 2;
+        $utf8 = 1 if $term eq 'charset' && $line =~ /^utf/;  # key off charset to decode value
+        $line = utf8::decode($line) if $utf8;
 
         # here string term-inator (har)
         if ($here) {
@@ -81,8 +91,9 @@ sub parse {
                 $term = $lterm;
             }
         } else {
-            $term =~ s/^(\s+)//;     # leading space
-            $s = length($1) + 1;     # counter+hack
+            # count leading space if it's there
+            $s = 1;     # reset
+            $s += length($1) if $term =~ s/^(\s+)//;
             $line =~ s/\s+$//;       # trailing space
 
             # uplevel pre-check (may have a value below)
@@ -124,31 +135,43 @@ sub parse {
             }
 
             my @val;
-            if ($term =~ /^js/ || $term eq 'messages') {
+            if ($term =~ /^js/ || $term =~ /^on[a-z]/ || $term eq 'messages' || $term eq 'comment') {
                 @val = $line;   # verbatim
             } elsif ($line =~ s/^\\(.)//) {
-                # reference - this is tricky
-                # go all the way up to the top to make sure
-                my $l=0;
-                1 while my $pkg = caller($l++);
-                $line = "${1}$pkg\::$line" unless $line =~ /::/;
+                # Reference - this is tricky. Go all the way up to
+                # the top to make sure, or use $self->{caller} if
+                # we were given a place to go.
+                my $r = $1;
+                my $l = 0;
+                my @p;
+                if ($self->{caller}) {
+                    @p = $self->{caller};
+                } else {
+                    while (my $pkg = caller($l++)) {
+                        push @p, $pkg;
+                    }
+                }
+                $line = "$r$p[-1]\::$line" unless $line =~ /::/;
+                debug 2, qq{eval "\@val = (\\$line)"};
                 eval "\@val = (\\$line)";
+                belch "Loading $line failed: $@" if $@;
             } else {
                 # split commas
                 @val = split /\s*,\s*/, $line;
 
                 # m=Male, f=Female -> [m,Male], [f,Female]
-                for (@val) {
-                    $_ = [ split /\s*=\s*/, $_, 2 ] if /=/;
+                for (my $i=0; $i < @val; $i++) {
+                    $val[$i] = [ split /\s*=\s*/, $val[$i], 2 ] if $val[$i] =~ /=/;
                 }
             }
 
             # only arrayref on multi values b/c FB is "smart"
             if ($ptr->{$term}) {
                 $ptr->{$term} = (ref $ptr->{$term})
-                                    ? [ @{$ptr->{$term}}, @val ] : @val > 1 ? \@val : $val[0];
+                                    ? [ @{$ptr->{$term}}, @val ] : @val > 1 ? \@val :
+                                      ref($val[0]) eq 'ARRAY' ? \@val : $val[0];
             } else {
-                $ptr->{$term} = @val > 1 ? \@val : $val[0];
+                $ptr->{$term} = @val > 1 ? \@val : ref($val[0]) eq 'ARRAY' ? \@val : $val[0];
             }
             $inval = 1;
         } else {
@@ -432,11 +455,11 @@ L<CGI::FormBuilder>, L<Text::FormBuilder>
 
 =head1 REVISION
 
-$Id: File.pm,v 1.23 2006/02/24 01:42:29 nwiger Exp $
+$Id: File.pm 100 2007-03-02 18:13:13Z nwiger $
 
 =head1 AUTHOR
 
-Copyright (c) 2005-2006 Nathan Wiger <nate@wiger.org>. All Rights Reserved.
+Copyright (c) L<Nate Wiger|http://nateware.com>. All Rights Reserved.
 
 This module is free software; you may copy this under the terms of
 the GNU General Public License, or the Artistic License, copies of
